#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <compiler:python, e.g. e20:p383b> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    setup_ups=$SETUP_UPS  # make sure ups does a full "setup"
    unset SETUP_UPS
    source `${UPS_DIR}/bin/ups setup ${setup_ups}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=pqxx
origpkgver=v6_2_5
pkgver=${origpkgver}e
ssibuildshims_version=v1_04_13
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//' | sed -e 's/a//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "debug" ]
then
    cflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

if [[ "${basequal}" == "e1[45]:p271[1345][abd]?" ]]
then
  cxxflg="${cflg} -std=c++14 -Wno-deprecated-declarations"
elif [[ "${basequal}" == "e1[79]:p271[45][ab]?" ]]
then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations"
elif [[ "${basequal}" == "c[27]:p271[45][ab]?" ]] 
then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations"
elif [[ "${basequal}" == "e1[79]:p372" ]]
then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations"
elif [[ "${basequal}" == "c[27]:p372" ]]
then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations"
elif [[ "${basequal}" == "e1[79]:p383b" ]]
then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations"
elif [[ "${basequal}" == "e2[0]:p383b" ]]
then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations"

elif [[ "${basequal}" == "c[27]:p383b" ]]
then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations"
elif [[ "${basequal}" == "e1[9]:p392" ]]
then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations"
elif [[ "${basequal}" == "e2[0]:p392" ]]
then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations"

elif [[ "${basequal}" == "c[7]:p392" ]]
then
  cxxflg="${cflg} -std=c++17 -Wno-deprecated-declarations"
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

if [ -z ${POSTGRESQL_VERSION} ]
then
   echo "ERROR: failed to setup postgresql"
   exit 1
else
   echo "building with postgresql version: ${POSTGRESQL_VERSION}"
fi

echo "Building ${package} in ${pkgdir}"

rm -rf ${pkgdir}
mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

set -x

# pqxx now requires an in source build
cd ${pkgdir}
tar -xf ${tardir}/${pkgtarname}.tar
# apply patch 
##patch -b -p0 < ${patchdir}/${pkgtarname}.patch || ssi_die "ERROR: failed to apply patch."

cd ${pkgdir}/${pkgtarname}

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

CXXFLAGS="${cxxflg}" CFLAGS="${cflg}" ./configure --prefix=${pkgdir} --enable-shared --disable-documentation || ssi_die "ERROR: configure failed."
make -j ${ncore} || ssi_die "ERROR: make failed."
# make check || ssi_die "ERROR: make check failed."
make install || ssi_die "ERROR: make install failed."
make clean || ssi_die "ERROR: make clean failed."

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "ERROR: setup ${package} ${pkgver} -q ${fullqual} failed"
echo "${package} is installed at ${PQXX_FQ_DIR}"

# fixing embedded path
mv ${PQXX_FQ_DIR}/bin/pqxx-config ${PQXX_FQ_DIR}/bin/pqxx-config.bak
cat ${PQXX_FQ_DIR}/bin/pqxx-config.bak | sed 's%^prefix=.*$%prefix=${PQXX_FQ_DIR}%' > ${PQXX_FQ_DIR}/bin/pqxx-config
chmod +x  ${PQXX_FQ_DIR}/bin/pqxx-config

# this must be last 
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

exit 0
